#ifndef __DCL_DATE_TIME_H__
#error "Never use <dcl/core/DateTime.inl> directly; include <dcl/core/DateTime.h>"
#endif

#ifndef __DCL_DATE_TIME_INL__
#define __DCL_DATE_TIME_INL__

/*
#if __DCL_DEBUG
static const char* __pszDateTime_inl__ = "dcl/DateTime.inl";
#undef __THIS_FILE__
#define __THIS_FILE__ __pszDateTime_inl__
#endif
*/

//////////////// Date ////////////////

DCLCAPI inline Date operator + (long nDays, const Date& d)
{
	return operator + (d, nDays);
}	

DCLCAPI inline Date operator - (const Date& d, long nDays)
{
	return operator + (d, -nDays);
}

DCLCAPI inline long operator - (const Date& d1, const Date& d2)
{
	return d1.days() - d2.days();
}

DCLCAPI inline bool operator == (const Date& d1, const Date& d2)
{
	return d1.days() == d2.days();
}

DCLCAPI inline bool operator != (const Date& d1, const Date& d2)
{
	return d1.days() != d2.days();
}

DCLCAPI inline bool operator >= (const Date& d1, const Date& d2)
{
	return d1.days() >= d2.days();
}

DCLCAPI inline bool operator <= (const Date& d1, const Date& d2)
{
	return d1.days() <= d2.days();
}

DCLCAPI inline bool operator > (const Date& d1, const Date& d2)
{
	return d1.days() > d2.days();
}

DCLCAPI inline bool operator < (const Date& d1, const Date& d2)
{
	return d1.days() < d2.days();
}

//////////////// Time ////////////////

inline long Time::totalMilliSeconds() const
{
	return m_uMilliSeconds;
}

DCLCAPI inline Time operator + (long nMilliSeconds, const Time& t)
{
	return operator + (t, nMilliSeconds);
}	

DCLCAPI inline Time operator - (const Time& t, long nMilliSeconds)
{
	return operator + (t, -nMilliSeconds);
}

DCLCAPI inline long operator - (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() - t2.totalMilliSeconds();
}

DCLCAPI inline bool operator == (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() == t2.totalMilliSeconds();
}

DCLCAPI inline bool operator != (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() != t2.totalMilliSeconds();
}

DCLCAPI inline bool operator >= (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() >= t2.totalMilliSeconds();
}

DCLCAPI inline bool operator <= (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() <= t2.totalMilliSeconds();
}

DCLCAPI inline bool operator > (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() > t2.totalMilliSeconds();
}

DCLCAPI inline bool operator < (const Time& t1, const Time& t2)
{
	return t1.totalMilliSeconds() < t2.totalMilliSeconds();
}

//////////////// Interval ////////////////
inline int64_t Interval::totalMilliSeconds() const
{
	return m_nMilliSeconds;
}

DCLCAPI inline bool operator == (const Interval& iv1, const Interval& iv2)
{
	return iv1.totalMilliSeconds() == iv2.totalMilliSeconds();
}

DCLCAPI inline bool operator != (const Interval& iv1, const Interval& iv2)
{
	return iv1.totalMilliSeconds() != iv2.totalMilliSeconds();
}

DCLCAPI inline bool operator >= (const Interval& iv1, const Interval& iv2)
{
	return iv1.totalMilliSeconds() >= iv2.totalMilliSeconds();
}

DCLCAPI inline bool operator <= (const Interval& iv1, const Interval& iv2)
{
	return iv1.totalMilliSeconds() <= iv2.totalMilliSeconds();
}

DCLCAPI inline bool operator > (const Interval& iv1, const Interval& iv2)
{
	return iv1.totalMilliSeconds() > iv2.totalMilliSeconds();
}

DCLCAPI inline bool operator < (const Interval& iv1, const Interval& iv2)
{
	return iv1.totalMilliSeconds() < iv2.totalMilliSeconds();
}

//////////////// DateTime ////////////////
inline Date& DateTime::date()
{
	return m_date;
}

inline const Date& DateTime::date() const
{
	return m_date;
}

inline Time& DateTime::time()
{
	return m_time;
}

inline const Time& DateTime::time() const
{
	return m_time;
}

DCLCAPI inline bool operator == (const DateTime& dt1, const DateTime& dt2)
{
	return DateTime::compare(dt1, dt2) == 0;
}

DCLCAPI inline bool operator != (const DateTime& dt1, const DateTime& dt2)
{
	return DateTime::compare(dt1, dt2) != 0;
}

DCLCAPI inline bool operator >= (const DateTime& dt1, const DateTime& dt2)
{
	return DateTime::compare(dt1, dt2) >= 0;
}

DCLCAPI inline bool operator <= (const DateTime& dt1, const DateTime& dt2)
{
	return DateTime::compare(dt1, dt2) <= 0;
}

DCLCAPI inline bool operator > (const DateTime& dt1, const DateTime& dt2)
{
	return DateTime::compare(dt1, dt2) > 0;
}

DCLCAPI inline bool operator < (const DateTime& dt1, const DateTime& dt2)
{
	return DateTime::compare(dt1, dt2) < 0;
}

/*
#if __DCL_DEBUG
#undef __THIS_FILE__
#define __THIS_FILE__ __T(__FILE__)
#endif
*/

#endif	// __DCL_DATE_TIME_INL__

